package com.isu.gaswellwatch.modbus.data.decode.listener;

import cn.hutool.core.map.MapUtil;
import cn.hutool.json.JSONUtil;
import com.isu.gaswellwatch.modbus.data.ModbusMessage;
import com.isu.gaswellwatch.modbus.data.PersistenceHandler;
import com.isu.gaswellwatch.modbus.data.decode.DecodeHandler;
import com.serotonin.modbus4j.msg.ReadInputRegistersResponse;
import com.serotonin.modbus4j.serial.rtu.RtuMessageParser;
import com.serotonin.modbus4j.serial.rtu.RtuMessageResponse;
import com.serotonin.modbus4j.sero.util.queue.ByteQueue;
import jakarta.annotation.Resource;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.amqp.core.BatchMessageListener;
import org.springframework.amqp.core.Message;
import org.springframework.data.redis.core.HashOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.jdbc.core.JdbcTemplate;

import java.time.Duration;
import java.util.*;

/**
 * @author <a href="mailto:shilong.wang@alpha-ess.com">王仕龙</a>
 * 2024/11/23 0:43
 */
@Slf4j
@RequiredArgsConstructor
@SuppressWarnings("all")
public class ModbusMessagePersistListener implements BatchMessageListener {

    public static final String COMMAND_CACHE = "modbus:command:";
    private static final String COMMAND_SQL = "select * from commands where id = ?";
    private static final String COMMAND_POINT_SQL = "select * from command_points where command_id = ?";

    @Resource
    private JdbcTemplate jdbcTemplate;
    @Resource(name = "redisTemplate")
    private RedisTemplate redisTemplate;
    @Resource
    private Map<String, DecodeHandler> decodeHandlerMap;

    @Override
    public void onMessage(Message message) {
        this.onMessageBatch(List.of(message));
    }

    @Override
    public void onMessageBatch(List<Message> messages) {
        Long commandId;
        String[] messageSplit;
        Map<String, Object> commandMap;
        String messageString, collectionMessage, commandString;
        Map<String, ModbusMessage.MessagePoint> messagePointMap;
        // 网关标识/设备标识/命令标识/采集下发时间/采集数据接收时间/采集数据体
        // 4B454E454E4731343030303030333538/1/10000/1732365690000/1732365691000/01042C07D0000
        for (Message message : messages) {
            messageString = new String(message.getBody());
            messageSplit = StringUtils.split(messageString, "/");
            if (messageSplit.length < 6) {
                log.error("非法数据: {}", messageString);
                continue;
            }

            collectionMessage = messageSplit[5];
            commandId = NumberUtils.createLong(messageSplit[2]);
            commandString = Objects.toString(this.redisTemplate.opsForValue().get(COMMAND_CACHE + commandId), "");
            commandMap = StringUtils.isNotEmpty(commandString) ? JSONUtil.parseObj(commandString) : null;
            if (ObjectUtils.isEmpty(commandMap)) {
                commandMap = this.jdbcTemplate.queryForMap(COMMAND_SQL, commandId);
                if (ObjectUtils.isNotEmpty(commandMap)) {
                    log.error("指令[{}]不存在，数据: {}", commandId, messageString);
                    continue;
                }
                this.redisTemplate.opsForValue()
                        .setIfAbsent(COMMAND_CACHE + commandId, JSONUtil.toJsonStr(commandMap), Duration.ofDays(1));
            }
            DecodeHandler decodeHandler = Optional.ofNullable(MapUtil.getStr(commandMap, "decode_handler_name"))
                    .map(this.decodeHandlerMap::get)
                    .orElse(null);
            if (Objects.isNull(decodeHandler)) {
                log.error("指令[{}]未配置解析器，数据: {}", commandId, messageString);
                continue;
            }
            messagePointMap = new HashMap<>();
            try {
                String address;
                int index = MapUtil.getInt(commandMap, "start_address"), stepSize = 0;
                ByteQueue byteQueue = new ByteQueue(collectionMessage);
                RtuMessageParser masterParser = new RtuMessageParser(true);
                RtuMessageResponse response = (RtuMessageResponse) masterParser.parseMessage(byteQueue);
                ReadInputRegistersResponse readInputRegistersResponse = (ReadInputRegistersResponse) response.getModbusResponse();
                for (short value : readInputRegistersResponse.getShortData()) {
                    stepSize = index * 4;
                    messagePointMap.put(StringUtils.leftPad(String.valueOf(index), 4, '0'),
                            ModbusMessage.MessagePoint.builder()
                                    .parseValue(String.valueOf(value))
                                    .originalValue(StringUtils.substring(collectionMessage, 6 + stepSize, 10 + stepSize))
                                    .build());
                    index++;
                }
            } catch (Exception e) {
                log.error("初始数据解析异常: {}", messageString, e);
                continue;
            }
            if (ObjectUtils.isNotEmpty(messagePointMap)) {
                decode(ModbusMessage.builder()
                        .commandId(commandId)
                        .message(collectionMessage)
                        .messagePointMap(messagePointMap)
                        .gatewayIdentifier(messageSplit[0])
                        .deviceId(NumberUtils.createLong(messageSplit[1]))
                        .receiveTime(NumberUtils.createLong(messageSplit[4]))
                        .collectionTime(NumberUtils.createLong(messageSplit[3]))
                        .queueName(message.getMessageProperties().getConsumerQueue())
                        .build());
            }
        }
    }


    public void decode(ModbusMessage modbusMessage) {
        Long commandId = modbusMessage.getCommandId();
        String cacheName = ModbusMessagePersistListener.COMMAND_CACHE + commandId + ":points";
        String cachePoints = Objects.toString(this.redisTemplate.opsForValue().get(cacheName), "");

        List<Map<String, Object>> commandPointList = StringUtils.isNotEmpty(cachePoints) ? (List) JSONUtil.parseArray(cachePoints) : null;
        if (ObjectUtils.isEmpty(commandPointList)) {
            commandPointList = this.jdbcTemplate.queryForList(COMMAND_POINT_SQL, commandId);
            if (ObjectUtils.isNotEmpty(commandPointList)) {
                log.error("指令[{}]点表配置为空", commandId, cachePoints);
                throw new RuntimeException("指令[" + commandId + "]点表未配置");
            }
            this.redisTemplate.opsForValue().setIfAbsent(cacheName, JSONUtil.toJsonStr(commandPointList), Duration.ofDays(1));
        }

        DecodeHandler decodeHandler;
        String fieldName, decodeName;
        int startAddress = 0, stepSize = 0;
        ModbusMessage.MessagePoint messagePoint;
        String deviceDataCacheName = PersistenceHandler.DEVICE_DATA_CACHE + modbusMessage.getDeviceId();
        HashOperations<String, Object, Object> hashOperations = this.redisTemplate.opsForHash();
        for (Map<String, Object> point : commandPointList) {
            fieldName = MapUtil.getStr(point, "field");
            stepSize = MapUtil.getInt(point, "step_size");
            decodeName = MapUtil.getStr(point, "decode_name");
            startAddress = MapUtil.getInt(point, "start_address");
            String value;
            if (stepSize <= 1) {
                messagePoint = modbusMessage.getMessagePointMap()
                        .get(StringUtils.leftPad(String.valueOf(startAddress), 4, '0'));
                decodeMessage(decodeName, messagePoint);
                value = messagePoint.getValue();
            } else {
                value = decodeCommandPoint(modbusMessage.getMessagePointMap(), decodeName,
                        MapUtil.getStr(point, "format"), startAddress, stepSize);
            }
            hashOperations.put(deviceDataCacheName, fieldName, value);
        }
    }

    private void decodeMessage(String decodeName, ModbusMessage.MessagePoint messagePoint) {
        if (StringUtils.isNotBlank(decodeName)) {
            DecodeHandler decodeHandler = this.decodeHandlerMap.get(decodeName);
            if (Objects.nonNull(decodeHandler)) {
                decodeHandler.decode(messagePoint);
            }
        }
    }

    private String decodeCommandPoint(Map<String, ModbusMessage.MessagePoint> pointMap,
                                      String decodeName, String format, int startAddress, int stepSize) {
        String[] values = new String[stepSize];
        ModbusMessage.MessagePoint messagePoint;
        for (int i = 0; i < stepSize; i++) {
            messagePoint = pointMap.get(StringUtils.leftPad(String.valueOf(startAddress + i), 4, '0'));
            decodeMessage(decodeName, messagePoint);
            values[i] = messagePoint.getValue();
        }
        return String.format(format, values);
    }

}
