package com.isu.gaswellwatch.service.impl;

import java.util.Collection;
import java.util.List;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.isu.gaswellwatch.config.SnowflakeConfig;
import com.isu.gaswellwatch.exception.BusinessException;
import com.isu.gaswellwatch.utils.ConverterUtil;
import jakarta.annotation.Resource;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.isu.gaswellwatch.dto.GasWellCreateRequest;
import com.isu.gaswellwatch.dto.GasWellEditRequest;
import com.isu.gaswellwatch.vo.GasWellVO;
import com.isu.gaswellwatch.dao.GasWellDao;
import com.isu.gaswellwatch.entity.GasWell;
import com.isu.gaswellwatch.service.GasWellService;
import com.isu.gaswellwatch.mapper.GasWellMapper;
import org.mapstruct.factory.Mappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

/**
 * 气井Service业务层处理
 * 
 * @author scwsl
 * @date 2024-11-17
 */
@Service
@RequiredArgsConstructor
@Transactional(readOnly = true)
public class GasWellServiceImpl  extends ServiceImpl<GasWellDao, GasWell> implements GasWellService {

    @Resource
    private SnowflakeConfig snowflakeConfig;

    private final GasWellDao gasWellDao;
    private final GasWellMapper gasWellMapper = Mappers.getMapper(GasWellMapper.class);

    /**
     * 查询气井
     * 
     * @param id 气井主键
     * @return 气井
     */
    @Override
    public GasWellVO selectGasWellById(Long id) {
        return gasWellDao.selectGasWellById(id);
    }

    /**
     * 查询气井列表
     * 
     * @param gasWell 气井
     * @return 气井
     */
    @Override
    public List<GasWellVO> selectGasWellList(GasWell gasWell) {
        return gasWellMapper.toPageVO(gasWellDao.selectGasWellList(gasWell));
    }

    /**
     * 分页查询气井列表
     *
     * @return 气井
     */
    @Override
    public Page<GasWellVO> pageForQuery(Integer currentPage, Integer pageSize, String name, Long blockId){
        return gasWellDao.pageForQuery(new Page<>(currentPage, pageSize), name,blockId);
    }

    /**
     * 新增气井
     * 
     * @param gasWellCreateRequest 气井
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Throwable.class)
    public int insertGasWell(GasWellCreateRequest gasWellCreateRequest) {
        GasWell gasWell = ConverterUtil.convert(gasWellCreateRequest,GasWell.class);

        //查重
        List<GasWell> list = list(new LambdaQueryWrapper<GasWell>().eq(GasWell::getName, gasWellCreateRequest.getName()));
        if(CollectionUtil.isNotEmpty(list)) {
            throw new BusinessException("已有相同名称，请重新输入");
        }
        gasWell.setId(snowflakeConfig.snowflakeId());
        return gasWellDao.insertGasWell(gasWell);
    }

    /**
     * 修改气井
     * 
     * @param gasWellEditRequest 气井
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Throwable.class)
    public int updateGasWell(GasWellEditRequest gasWellEditRequest) {
        GasWellVO gasWellVO = ConverterUtil.convert(gasWellEditRequest,GasWellVO.class);
        return gasWellDao.updateGasWell(gasWellVO);
    }

    /**
     * 删除气井信息
     * 
     * @param id 气井主键
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Throwable.class)
    public int deleteGasWellById(Long id) {
        //判断气井是否被关联设备，如果已经关联设备则无法删除
        GasWellVO gasWellVO = gasWellDao.selectGasWellById(id);
        if(gasWellVO.getDeviceId() != null){
            throw new BusinessException("该气井已经关联设备，无法删除");
        }
        return gasWellDao.deleteGasWellById(id);
    }

    /**
     * 批量删除删除气井信息
     *
     * @param ids 气井主键
     * @return 数量
     */
    @Override
    @Transactional(rollbackFor = Throwable.class)
    public int deleteGasWellByIds(Collection<Long> ids) {
        return gasWellDao.deleteGasWellByIds(ids);
    }

}
